// 示例：一个通过加法实现非负整数乘法计算的函数
//要点：
   // 用以说明演绎推理进行验证程序的过程
//关键字：验证过程说明，虚拟变量，循环不变式
#define N 50
/*@ requires 0 <= n < N && 0 <= m < N*n; // m, n 为非负整数，m和n不能同时为零
                                         // N取有限值50用于防止计算溢出，简化溢出处理
    ensures  \result == m * n; 
*/
int multadd(const int m, const int n)
{
	int x, y; 

	x = 0;	// x原来的值用%1x表示
	y = 0;	// y原来的值用%1y表示
    
    /*@ loop invariant // 循环不变式
        0 <= y <= n && x == m*y && 0 <= n < N && 0 <= m < N*n;
    */ 
	while (y < n){	 
		x = x + m; // x原来的值用%2x表示，则有%2x == m*y && x == %2x+m

        y = y + 1; // y原来的值用%2y表示，则有y == %2y+1
                   // 注意，为简化计算，验证器在实现时不生成%2y，而是直接用y-1替代

        // x现值与x旧值的关系及x旧值与y现值间的关系：x == %2x+m && %2x == m*(y-1) 
        // 在循环体结束点x和y的现值之间的关系为：x == m*(y-1)+m，即有 x == m*y
	}
    // 循环结束时有：y == n && x == m*y，即 x == m*n
	return x;  // 函数返回值为 \result == x，即 \result == m*n。它蕴含函数协议的后件
}

/////////////////////////////////////////////////////////////////////////////////
// 另一个加法实现乘法计算的函数
//要点：
   // 考虑一般的情况和计算溢出时的处理
//关键字：循环不变式，数值溢出，异常退出
#include <stdlib.h>
#include <limits.h>

/*@ requires
      0 <= n <= INT_MAX && 0 <= m <= INT_MAX;
    exits \exit_status == 1;  // 异常退出，状态值为1
    ensures \result == m * n;
*/
int multadd2(const int m, const int n){
    int x, y;
    x = 0 ;
    y = 0 ;
    /*@ loop invariant
        0 <= y <= n && x == m*y && 0 <= n <= INT_MAX && 0 <= m <= INT_MAX;
    */
    while (y < n){
        if (x <= INT_MAX - m){
            x = x + m ;
            y = y + 1 ;
         }else{
             exit(1); // 计算溢出了
         }
    }
    return x;
}